@@ -1,3 +1,3 @@
-from .client import Client, generate_token
+from .client import Client, setup_logging
 from .Datamodels import *
 from .exceptions import *
